#include "Platform/Platform.hpp"
#include <Agents/Agent.hpp>
#include <Agents/Perception/Perception.hpp>
#include <Environment/PictureEnvironment.hpp>

const sf::Vector2f DEFAUMT_SIMZE = sf::Vector2f(800, 600);
int PERCEPTION_SIZE = 4;

PictureEnvironment::PictureEnvironment()
{
}

PictureEnvironment::PictureEnvironment(std::string& iFilePath, sf::RenderWindow* iWindow, util::Platform* iPlatform) :
	fSize(iWindow->getSize())
{

	fFilePath = iFilePath;
	//sf::Texture fLoadedImage;
	if (!fTextureImage.loadFromFile(fFilePath))
	{
		std::cout << "\n COULDNT LOAD PICTURE\n";
	}
	fImage = fTextureImage.copyToImage();

	//memccpy(&fFilePath, &iFilePath, *iFilePath., 10);
	fPlatform = iPlatform;
	fWindow = iWindow;
	float screenScalingFactor = fPlatform->getScreenScalingFactor(fWindow->getSystemHandle());
	fWindow->create(sf::VideoMode(800.0f * screenScalingFactor, 600.0f * screenScalingFactor), "SFML works!");
	fPlatform->setIcon(fWindow->getSystemHandle());
	fInitialSize.x = 800.0f * screenScalingFactor;
	fInitialSize.y = 600.0f * screenScalingFactor;
	//fShape = sf::RectangleShape(fSize);
	fShape = sf::RectangleShape(DEFAUMT_SIMZE);

	fShape.setFillColor(sf::Color::White);

	fShape.setTexture(&fTextureImage);
}

Perception PictureEnvironment::getPerception(int iX, int iY)
{
	Perception vPerception = Perception(PERCEPTION_SIZE, PERCEPTION_SIZE);
	iX++;
	iY++;
	return vPerception;
}

bool PictureEnvironment::loop()
{
	if (fWindow->isOpen())
	{
		while (fWindow->pollEvent(fEvent))
		{
			if (fEvent.type == sf::Event::Closed)
				fWindow->close();
		}
		// fAgentsIterator = fAgents.begin();
		std::list<Agent>::iterator vAgentsIterator = fAgents.begin();
		fAgents.begin();
		while (vAgentsIterator != fAgents.end())
		{

			//std::cout << "loop" << vAgentsIterator->fColor->toInteger() << " at " << &vAgentsIterator << "\n";
			vAgentsIterator->loop();
			vAgentsIterator++;
		}
		fTextureImage.loadFromImage(fImage);
		fShape.setTexture(&fTextureImage);
		fWindow->clear();
		fWindow->draw(fShape);
		fWindow->display();
	}
	else
	{
		std::cout << "\nBah on ferme";
		return false;
	}
	return true;
}

int PictureEnvironment::getHeight()
{
	return fImage.getSize().y;
}

int PictureEnvironment::getWidth()
{
	return fImage.getSize().x;
}

void PictureEnvironment::makeAgents(int iNumber)
{

	fAgents = std::list<Agent>();

	for (int i = 0; i < iNumber; i++)
	{
		Agent vTmpAgent = Agent(this);
		fAgents.push_back(vTmpAgent);
		//std::cout << "pushback" << vTmpAgent.fColor.toInteger() << " at " << &vTmpAgent << "\n"
		//		  << " size : " << fAgents.size() << "\n";
	}
	std::list<Agent>::iterator vAgentsIterator = fAgents.begin();
	while (vAgentsIterator != fAgents.end())
	{

		//std::cout << "loop" << vAgentsIterator->fColor->toInteger() << " at " << &vAgentsIterator << "\n";
		vAgentsIterator++;
	}
}

void PictureEnvironment::displayLockInput()
{
	fWindow->create(sf::VideoMode(fInitialSize.x, fInitialSize.y), "Last peek");
	fWindow->clear();
	fWindow->draw(fShape);
	fWindow->display();
	int i;
	std::cin >> i;
}

void PictureEnvironment::paint(int iX, int iY, const sf::Color iColour)
{
	//std::cout << "juste avant : " << iX << " et " << iY;
	fImage.setPixel(iX, iY, iColour);
}